<?php
// $Id: moodle.cckfield.inc,v 1.4 2010/08/02 23:08:01 chris Exp $

/**
 * @file
 * Moodle Module assists with integrating a Moodle instance with a Drupal
 * instance.  This file provides a CCK field which displays Moodle course
 * data.
 *
 * NOTE!  Still in development, and not yet fully functional.
 * Patches gratefully accepted.
 */

/**
 * Implementation of cck hook_field_info()
 */
function moodle_field_info() {
/* DEBUG */
#drupal_set_message(__FUNCTION__);
/* END DEBUG */
  return array(
    'computed' => array(
      'label' => t('Computed'),
      'description' => t('Create field data via PHP code.'),
      'callbacks' => array(
        'tables' => CONTENT_CALLBACK_DEFAULT,
        'arguments' => CONTENT_CALLBACK_DEFAULT,
        ),
      ),
    );
}

/**
 * Implementation of cck hook_field_settings()
 */
function moodle_field_settings($op, $field) {
/* DEBUG */
#drupal_set_message(__FUNCTION__);
/* END DEBUG */
  switch ($op) {
    case 'form':
      $form = array();
      // these next 3 have been moved from widget to field, so they copy default values from widget
      $form['code'] = array(
        '#type' => 'textarea',
        '#rows' => 15,
        '#title' => t('Computed Code'),
        '#description' => t('The variables available to your code are: ') .'<code>&amp;$node, $field, and &amp;$node_field</code>'. t('. To set the value of the field, set ') .'<code>$node_field[0][\'value\']</code>'. t('. Here\'s a simple example which sets the computed field\'s value to the value of the sum of the number fields field_a and field_b: ') .'<code>$node_field[0][\'value\'] = $node->field_a[0][\'value\'] + $node->field_b[0][\'value\'];</code>',
        '#default_value' => !empty($field['code']) ? $field['code'] : '$node_field[0][\'value\'] = "";',
      );
      $form['display'] = array(
        '#type' => 'checkbox',
        '#title' => t('Display this field'),
        '#default_value' => is_numeric($field['code']) ? $field['code'] : TRUE ,
      );
      $form['display_format'] = array(
        '#type' => 'textarea',
        '#title' => t('Display Format'),
        '#description' => t('This code should assign a string to the $display variable, which will be printed as the value of the field. The stored value of the field is in $node_field_item[\'value\'].  Note: this code has no effect if you use the "Computed Value" formatter option.'),
        '#default_value' => !empty($field['display_format']) ? $field['display_format'] : '$display = $node_field_item[\'value\'];',
      );
      $form['store'] = array(
        '#type' => 'checkbox',
        '#title' => t('Store using the database settings below'),
        '#default_value' => is_numeric($field['store']) ? $field['store'] : 1 ,
      );
      $form['database'] = array('#type' => 'fieldset', '#title' => t('Database Storage Settings'));
      $form['database']['data_type'] = array(
        '#type' => 'radios',
        '#title' => t('Data Type'),
        '#description' => t('The SQL datatype to store this field in.'),
        '#default_value' => !empty($field['data_type']) ? $field['data_type'] : 'varchar',
        '#options' => array('int' => 'int', 'float' => 'float', 'varchar' => 'varchar', 'text' => 'text', 'longtext' => 'longtext'),
        '#required' => FALSE,
      );
      $form['database']['data_length'] = array(
        '#type' => 'textfield',
        '#title' => t('Data Length'),
        '#default_value' => !empty($field['data_length']) ? $field['data_length'] : NULL,
        '#required' => FALSE,
      );
      $form['database']['data_default'] = array(
        '#type' => 'textfield',
        '#title' => t('Default Value'),
        '#default_value' => $field['data_default'],
        '#required' => FALSE,
      );
      $form['database']['data_not_NULL'] = array(
        '#type' => 'checkbox',
        '#title' => t('Not NULL'),
        '#default_value' => is_numeric($field['data_not_NULL']) ? $field['data_not_NULL'] : FALSE,
      );
      $form['database']['data_sortable'] = array(
        '#type' => 'checkbox',
        '#title' => t('Sortable'),
        '#default_value' => is_numeric($field['data_sortable']) ? $field['data_sortable'] : TRUE,
      );
      return $form;
    case 'validate':
      if ($field['store']) {
        if (empty($field['data_type'])) {
          form_set_error('data_type', t('To store this field in the database, please specify a data type.'));
        }
        if (!($field['data_type'] == 'text' || $field['data_type'] == 'longtext') && empty($field['data_length'])) {
          form_set_error('data_length', t('To store this field in the database, please specify the data length.'));
        }
      }
      break;
    case 'save':
      return array('code', 'display', 'display_format', 'store', 'data_type', 'data_length', 'data_not_NULL', 'data_default', 'data_sortable');
    case 'database columns':
      if ($field['store']) {
        $columns = array('value' => array());
        $columns['value']['type'] = isset($field['data_type']) ? $field['data_type'] : 'varchar';
        // text and longtext should not have a length, so we ignore it
        if (!($field['data_type'] == 'text' || $field['data_type'] == 'longtext')) {
          $columns['value']['length'] = isset($field['data_length']) ? $field['data_length'] : 32;
        }
        $columns['value']['not NULL'] = isset($field['data_not_NULL']) ? $field['data_not_NULL'] : TRUE;
        $columns['value']['sortable'] = isset($field['data_sortable']) ? $field['data_sortable'] : FALSE;
        if ($field['data_default'] != '')  {
          $columns['value']['default'] = $field['data_default'];
        }
      }
      return $columns;

    case 'filters':
      return array(
        'default' => array(
          'name' => t('Default'),
          'operator' => 'views_handler_operator_gtlt',
        ),
      );
    case 'callbacks':
      return array(
        'view' => CONTENT_CALLBACK_CUSTOM,
      );
    case 'views data':
      $allowed_values = content_allowed_values($field);
      if (count($allowed_values)) {
        $data = content_views_field_views_data($field);
        $db_info = content_database_info($field);
        $table_alias = content_views_tablename($field);

        // Swap the filter handler to the 'in' operator.
        $data[$table_alias][$field['field_name'] .'_value']['filter']['handler'] = 'content_handler_filter_many_to_one';
        return $data;
      }
  }
}

function _moodle_compute_value(&$node, $field, &$node_field) {
/* DEBUG */
#drupal_set_message(__FUNCTION__ .' calling REST interface');
/* END DEBUG */
//  _moodle_page_theme_course_list(_moodle_get_courses());
  $node_field[0]['value'] =  _moodle_page_theme_course_list(_moodle_get_courses());
  return;
  /*
  if (isset($field['code'])) {
    eval($field['code']);
  }
  */
}

/**
 * Implementation of cck hook_field()
 */
function moodle_field($op, &$node, $field, &$node_field, $teaser, $page) {
/* DEBUG */
#drupal_set_message(__FUNCTION__ . " op = '$op'");
/* END DEBUG */
  switch ($op) {
    case 'load':
      // compute field on load if it isn't stored in the database
      if (!$field['store']) {
        _moodle_compute_value($node, $field, $node_field);// alters $node_field
        return array($field['field_name'] => $node_field);
      }
      break;
    case 'view':
/* DEBUG */
#drupal_set_message(__FUNCTION__. " VIEWING");
/* END DEBUG */
      if ($field['display']) {
        $items = array();
        foreach ($node_field as $delta => $item) {
          $items[$delta]['view'] = content_format($field, $item, 'default', $node);
        }
        return theme('field', $node, $field, $items, $teaser, $page);
      }
      break;
    case 'validate':
      break;
    case 'insert':
    case 'update':
/* DEBUG */
#drupal_set_message(__FUNCTION__. " INSERT or UPDATE");
/* END DEBUG */
      _moodle_compute_value($node, $field, $node_field);  // alters $node_field
      break;
  }
}

/**
 * Implementation of cck hook_widget_info()
 */
function moodle_widget_info() {
  return array(
    'computed' => array(
      'label' => t('Computed'),
      'field types' => array('computed'),
      'multiple values' => CONTENT_HANDLE_MODULE,
    ),
  );
}

/**
 * Implementation of cck hook_widget()
 */
function moodle_widget(&$form, &$form_state, $field, $items, $delta = 0) {
/* DEBUG */
#drupal_set_message(__FUNCTION__ .' called with its weird foreach loop');
/* END DEBUG */
  $elements = array();
  foreach ($items as $delta => $item) {
    $elements[$delta]['value'] = array(
      '#type' => 'value',
      '#tree' => TRUE,
      '#default_value' => isset($item['value']) ? $item['value'] : NULL,
    );
  }
  return $elements;
}

/**
 * Implementation of cck hook_view_item (obsolete, retained for backward compatibility with older cck)
 */
function moodle_view_item($field, $node_field_item, $node = NULL) {
/* DEBUG */
#drupal_set_message(__FUNCTION__ .' called OBSOLETE');
/* END DEBUG */
  global $base_url;
  if ($field['display']) {
    $display = '';
    eval($field['display_format']);
    return $display;
  }
}

/**
 * Implementation of hook_theme().
 */
function moodle_theme() {
/* DEBUG */
#drupal_set_message(__FUNCTION__ .' called SETTING FORMATTERS!');
/* END DEBUG */
  return array(
    'moodle_formatter_default' => array(
      'arguments' => array('element' => NULL),
    ),
    'moodle_formatter_plain' => array(
      'arguments' => array('element' => NULL),
    ),
    'moodle_formatter_markup' => array(
      'arguments' => array('element' => NULL),
    ),
    'moodle_formatter_computed_value' => array(
      'arguments' => array('element' => NULL),
    ),
  );
}

/**
 * Implementation of cck hook_field_formatter_info()
 */
function moodle_field_formatter_info() {
/* DEBUG */
#drupal_set_message(__FUNCTION__ .' called GETTING FORMATTER info');
/* END DEBUG */
  return array(
    'default' => array(
      'label' => t('Raw text'),
      'field types' => array('computed'),
    ),
    'plain' => array(
      'label' => t('Plain text'),
      'field types' => array('computed'),
    ),
    'markup' => array(
      'label' => t('Markup'),
      'field types' => array('computed'),
    ),
    'computed_value' => array(
      'label' => t('Computed Value'),
      'field types' => array('computed'),
    ),
  );
}

/**
 * Theme function for 'default' text field formatter.
 */
function theme_moodle_formatter_default($element) {
/* DEBUG */
#drupal_set_message(__FUNCTION__);
/* END DEBUG */
  $field = content_fields($element['#field_name']);
  // For "some" backwards compatibility
  $node_field_item['value'] = $element['#item']['value'];
  eval($field['display_format']);
  return $display;
}

/**
 * Theme function for 'plain' text field formatter.
 */
function theme_moodle_formatter_plain($element) {
/* DEBUG */
#drupal_set_message(__FUNCTION__);
/* END DEBUG */
  $field = content_fields($element['#field_name']);
  // For "some" backwards compatibility
  $node_field_item['value'] = $element['#item']['value'];
  eval($field['display_format']);
  return check_plain($display);
}

/**
 * Theme function for 'markup' text field formatter.
 */
function theme_moodle_formatter_markup($element) {
/* DEBUG */
#drupal_set_message(__FUNCTION__);
/* END DEBUG */
  $field = content_fields($element['#field_name']);
  // For "some" backwards compatibility
  $node_field_item['value'] = $element['#item']['value'];
  eval($field['display_format']);
  return check_markup($display);
}

/**
 * Theme function for 'computed_value' text field formatter.
 */
function theme_moodle_formatter_computed_value($element) {
/* DEBUG */
#drupal_set_message(__FUNCTION__);
/* END DEBUG */
  return $element['#item']['value'];
}

/**
 * Implementation of cck hook_content_is_empty().
 */
function moodle_content_is_empty() {
/* DEBUG */
#drupal_set_message(__FUNCTION__);
/* END DEBUG */
  return FALSE;
}

/**
 * Implementation of hook_token_list().
 */
function moodle_token_list($type = 'all') {
/* DEBUG */
#drupal_set_message(__FUNCTION__);
/* END DEBUG */
  if ($type == 'field' || $type == 'all') {
    $tokens = array();

    $tokens['moodle']['raw']       = t("Raw, unfiltered text.");
    $tokens['moodle']['formatted'] = t("Formatted and filtered text.");

    return $tokens;
  }
}

/**
 * Implementation of hook_token_values().
 */
function moodle_token_values($type, $object = NULL, $options = array()) {
/* DEBUG */
#drupal_set_message(__FUNCTION__);
/* END DEBUG */
  if ($type == 'field') {
    $item = $object[0];
    $tokens['raw']  = $item['value'];
    $tokens['formatted'] = $item['view'];
    return $tokens;
  }
}

require_once('moodle.pages.inc');
require_once('moodle.rest.inc');
